==> 0.1.1.tar.gz: lib/query/query_pageable.dart <==
import 'package:flutter/material.dart';
import 'package:stateful_provider/model/page_request.dart';
import 'package:stateful_provider/provider/base_provider.dart';

class QueryPageable<Model, Query> {
  final BaseProvider provider;
  final Future<List<Model>> Function(QueryPageRequest<Query> query) call;

  QueryPageable({
    @required this.provider,
    @required this.call,
    @required Query query,
  }) {
    changeQuery(query);
  }

  Query _query;
  int _next;
  int _current;
  bool _hasNext;
  List<Model> data = [];

  Future changeQuery(Query query) {
    _query = query;
    return refresh();
  }

  void clear() {
    _hasNext = true;
    _next = 0;
    _current = null;
    data.clear();
    provider.rebuild();
  }

  QueryPageRequest<Query> get _currentQuery =>
      QueryPageRequest<Query>(page: _current, query: _query);

  Future _fetch({@required bool refresh}) async {
    if (refresh) {
      clear();
    }

    if (!_hasNext && _current != _next) return;

    _current = _next;

    final response = await provider.enqueue(() => call(_currentQuery));

    if (response == null) return;

    _hasNext = response.isNotEmpty;
    if (response.isNotEmpty) {
      _next += 1;
      data.addAll(response);
      provider.rebuild();
    }
  }

  Future next() => _fetch(refresh: false);

  Future refresh() => _fetch(refresh: true);

  Widget list({
    @required
        Widget Function(BuildContext context, int index, Model model) builder,
    int fetchWhenLastVisible = 8,
    bool shrinkWrap = false,
  }) =>
      ListView.builder(
        itemCount: data.length,
        shrinkWrap: shrinkWrap,
        itemBuilder: (context, index) {
          if (data.length - index == fetchWhenLastVisible) {
            _fetch(refresh: false);
          }
          return builder(context, index, data[index]);
        },
      );
}

==> 0.3.0.tar.gz: lib/query/query_pageable.dart <==
import 'package:flutter/material.dart';
import 'package:stateful_provider/model/page_request.dart';
import 'package:stateful_provider/provider/base_provider.dart';

class QueryPageable<Model, Query> {
  BaseProvider provider;
  Future<List<Model>> Function(QueryPageRequest<Query> query) call;

  QueryPageable(
    this.provider,
    this.call,
    this._query,
  ) {
    refresh();
  }

  Query _query;
  int _next = 0;
  bool _hasNext = true;
  int? _current;
  List<Model> data = [];

  Future changeQuery(Query query) {
    _query = query;
    return refresh();
  }

  void clear() {
    _hasNext = true;
    _next = 0;
    _current = null;
    data.clear();
    provider.rebuild();
  }

  Future _fetch(bool refresh) async {
    if (refresh) {
      clear();
    }

    if (!_hasNext && _current != _next) return;

    _current = _next;

    final response = await provider
        .enqueue(() => call(QueryPageRequest<Query>(_current!, _query)));

    if (response == null) return;

    _hasNext = response.isNotEmpty;
    if (response.isNotEmpty) {
      _next += 1;
      data.addAll(response);
      provider.rebuild();
    }
  }

  Future next() => _fetch(false);

  Future refresh() => _fetch(true);

  Widget list(
    Widget Function(BuildContext context, int index, Model model) builder, {
    int fetchWhenLastVisible = 8,
    bool shrinkWrap = false,
  }) =>
      ListView.builder(
        itemCount: data.length,
        shrinkWrap: shrinkWrap,
        itemBuilder: (context, index) {
          if (data.length - index == fetchWhenLastVisible) {
            _fetch(false);
          }
          return builder(context, index, data[index]);
        },
      );
}
